@@ -1,6 +1,6 @@
 package Test::Trap::Builder::SystemSafe;
 
-use version; $VERSION = qv('0.1.2');
+use version; $VERSION = qv('0.2.0');
 
 use strict;
 use warnings;
@@ -17,18 +17,7 @@ sub import {
       $self->Exception("SystemSafe only works with real file descriptors; aborting");
     }
     my ($fh, $file) = tempfile( UNLINK => 1 ); # XXX: Test?
-    binmode $fh; # superfluos?
-    open my $fh_keeper, ">&$fileno"
-      or $self->Exception("Cannot dup '$fileno' for $name: '$!'");
-    my $autoflush_keeper = $globref->autoflush;
-    _close_reopen($self, $globref, $fileno, ">>$file",
-		  sub {
-		    sprintf "Cannot open %s for %s: '%s'",
-		      $file, $name, $!;
-		  },
-		 );
-    binmode *$globref; # must write with the same mode as we read.
-    $globref->autoflush(1);
+    my ($fh_keeper, $autoflush_keeper);
     $self->Teardown($_) for sub {
       if ($pid == $$) {
 	# this process opened it, so it gets to collect the contents:
@@ -36,17 +25,32 @@ sub import {
 	$self->{$name} .= $fh->getline;
 	close $fh; # don't leak this one either!
       }
+      close *$globref;
+      return unless $fh_keeper;
       # close and reopen the file to the keeper!
       my $fno = fileno $fh_keeper;
-      _close_reopen($self, $globref, $fileno, ">&$fno",
-		    sub {
-		      sprintf "Cannot dup '%s' for %s: '%s'",
-			fileno $fh_keeper, $name, $!;
-		    },
+      _close_reopen( $self, $globref, $fileno, ">&$fno",
+		     sub {
+		       close $fh_keeper;
+		       sprintf "Cannot dup '%s' for %s: '%s'",
+			 $fno, $name, $!;
+		     },
 		   );
       close $fh_keeper; # another potential leak, I suppose.
       $globref->autoflush($autoflush_keeper);
     };
+    binmode $fh; # superfluos?
+    open $fh_keeper, ">&$fileno"
+      or $self->Exception("Cannot dup '$fileno' for $name: '$!'");
+    $autoflush_keeper = $globref->autoflush;
+    _close_reopen( $self, $globref, $fileno, ">>$file",
+		   sub {
+		     sprintf "Cannot open %s for %s: '%s'",
+		       $file, $name, $!;
+		   },
+		 );
+    binmode *$globref; # must write with the same mode as we read.
+    $globref->autoflush(1);
     $self->Next;
   };
 }
@@ -86,7 +90,7 @@ Test::Trap::Builder::SystemSafe - "Safe" output layer backend using File::Temp
 
 =head1 VERSION
 
-Version 0.1.2
+Version 0.2.0
 
 =head1 DESCRIPTION
 
